/**
 * SCA4J
 * Copyright (c) 2009 - 2099 Service Symphony Ltd
 *
 * Licensed to you under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.  A copy of the license
 * is included in this distrubtion or you may obtain a copy at
 *
 *    http://www.opensource.org/licenses/apache2.0.php
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * This project contains code licensed from the Apache Software Foundation under
 * the Apache License, Version 2.0 and original code from project contributors.
 *
 *
 * Original Codehaus Header
 *
 * Copyright (c) 2007 - 2008 fabric3 project contributors
 *
 * Licensed to you under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.  A copy of the license
 * is included in this distrubtion or you may obtain a copy at
 *
 *    http://www.opensource.org/licenses/apache2.0.php
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * This project contains code licensed from the Apache Software Foundation under
 * the Apache License, Version 2.0 and original code from project contributors.
 *
 * Original Apache Header
 *
 * Copyright (c) 2005 - 2006 The Apache Software Foundation
 *
 * Apache Tuscany is an effort undergoing incubation at The Apache Software
 * Foundation (ASF), sponsored by the Apache Web Services PMC. Incubation is
 * required of all newly accepted projects until a further review indicates that
 * the infrastructure, communications, and decision making process have stabilized
 * in a manner consistent with other successful ASF projects. While incubation
 * status is not necessarily a reflection of the completeness or stability of the
 * code, it does indicate that the project has yet to be fully endorsed by the ASF.
 *
 * This product includes software developed by
 * The Apache Software Foundation (http://www.apache.org/).
 */
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.sca4j.fabric.services.contribution.processor;

import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Map;

import org.oasisopen.sca.annotation.EagerInit;
import org.oasisopen.sca.annotation.Reference;
import org.sca4j.host.contribution.ContributionException;
import org.sca4j.introspection.DefaultIntrospectionContext;
import org.sca4j.introspection.IntrospectionContext;
import org.sca4j.introspection.xml.Loader;
import org.sca4j.introspection.xml.LoaderException;
import org.sca4j.scdl.ValidationContext;
import org.sca4j.spi.services.contribution.Action;
import org.sca4j.spi.services.contribution.Contribution;
import org.sca4j.spi.services.contribution.ContributionManifest;
import org.sca4j.spi.services.contribution.ContributionProcessor;
import org.sca4j.spi.services.contribution.Resource;
import org.sca4j.spi.services.contribution.ResourceProcessor;

/**
 * The base class for ContributionProcessor implementations
 *
 * @version $Rev: 3672 $ $Date: 2008-04-19 03:49:29 +0100 (Sat, 19 Apr 2008) $
 */
@EagerInit
public abstract class AbstractContributionProcessor implements ContributionProcessor {
    
    @Reference public Map<String, ResourceProcessor> resourceProcessors;
    @Reference public Loader loader;

    /**
     * {@inheritDoc}
     * @see org.sca4j.spi.services.contribution.ContributionProcessor#process(org.sca4j.spi.services.contribution.Contribution, org.sca4j.scdl.ValidationContext, java.lang.ClassLoader)
     */
    public final void process(Contribution contribution, ValidationContext context, ClassLoader loader) throws ContributionException {
        ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        URI contributionUri = contribution.getUri();
        try {
            Thread.currentThread().setContextClassLoader(loader);
            for (Resource resource : contribution.getResources()) {
                if (!resource.isProcessed()) {
                    ResourceProcessor resourceProcessor = getResourceProcessor(resource.getUrl());
                    if (resourceProcessor != null) {
                        resourceProcessor.process(contributionUri, resource, context, loader);
                    }
                }
            }
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * {@inheritDoc}
     * @see org.sca4j.spi.services.contribution.ContributionProcessor#index(org.sca4j.spi.services.contribution.Contribution, org.sca4j.scdl.ValidationContext)
     */
    public final void index(Contribution contribution, final ValidationContext context) throws ContributionException {
        iterateArtifacts(contribution, new Action() {
            public void process(Contribution contribution, String contentType, URL url) throws ContributionException {
                ResourceProcessor resourceProcessor = getResourceProcessor(url);
                if (resourceProcessor != null) {
                    resourceProcessor.index(contribution, url, context);
                }
            }
        });
    }

    /**
     * {@inheritDoc}
     * @see org.sca4j.spi.services.contribution.ContributionProcessor#processManifest(org.sca4j.spi.services.contribution.Contribution, org.sca4j.scdl.ValidationContext)
     */
    public final void processManifest(Contribution contribution, final ValidationContext context) throws ContributionException {
        
        ContributionManifest manifest;
        try {
            URL manifestURL = getManifestUrl(contribution);
            ClassLoader cl = getClass().getClassLoader();
            URI uri = contribution.getUri();
            IntrospectionContext childContext = new DefaultIntrospectionContext(cl, uri, null);
            manifest = loader.load(manifestURL, ContributionManifest.class, childContext);
            if (childContext.hasErrors()) {
                context.addErrors(childContext.getErrors());
            }
            if (childContext.hasWarnings()) {
                context.addWarnings(childContext.getWarnings());
            }
        } catch (LoaderException e) {
            if (e.getCause() instanceof FileNotFoundException) {
                manifest = new ContributionManifest();
            } else {
                throw new ContributionException(e);
            }
        } catch (MalformedURLException e) {
            manifest = new ContributionManifest();
        }
        contribution.setManifest(manifest);
        
    }

    /**
     * Iterates through a list of artifacts within the contribution.
     * 
     * @param contribution Contribution.
     * @param action Action to be performed on each artifact.
     * @throws ContributionException If there is an unexpected exception.
     */
    protected abstract void iterateArtifacts(Contribution contribution, Action action) throws ContributionException;
    
    /**
     * Gets the manifest URLfor the contribution.
     * 
     * @param contribution Contribution.
     * @return Manifest URL for the contribution.
     * @throws MalformedURLException If unable to construct the manifest URL.
     */
    protected abstract URL getManifestUrl(Contribution contribution) throws MalformedURLException;
    
    /*
     * Gets the resource processor for the extension.
     */
    private ResourceProcessor getResourceProcessor(URL resourceUrl) {
        String externalForm = resourceUrl.toExternalForm();
        String type = externalForm.substring(externalForm.lastIndexOf('.') + 1);
        return resourceProcessors.get(type);
    }

}
